/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
EEPROM Algorithms for LiveLight V3 Firmware
Written by Henri Rantanen

Date		28.4.2014
Version		3.0.1
Target		PCB Version 5, Atmel ATMEGA32-16U

************************************************************************************/
#include "settings.h"

void WriteEE_defaults(void)
{
	// Disable all interrupts while writing EEPROM
	cli();	
	
	// Write EEPROM
	eeprom_update_byte ((uint8_t*)	0x00, HEADER);	// Write Check byte
	eeprom_update_byte ((uint8_t*)	0x01, (BAUD/100));	// Write BAUD rate
	eeprom_update_byte ((uint8_t*)	0x02, PREFIX);	// Write Prefix
	eeprom_update_byte ((uint8_t*)	0x03, SLEEP_DELAY );	// Write Shut-Down Delay
	eeprom_update_byte ((uint8_t*)	0x04, BLACKLEVEL	);	// Write BlackLevel
	eeprom_update_byte ((uint8_t*)	0x05, MODE	);	// Write Mode
	
	// Write black as default color to memory
	Color(0,0,0);
	WriteEE_Color();
		
	// Verify EEPROM
	if (EE_HEADER != HEADER | EE_BAUD != (BAUD/100) | EE_PREFIX != PREFIX | EE_SLEEP_DELAY != SLEEP_DELAY | EE_BLACK_LEVEL != BLACKLEVEL | EE_MODE != MODE)
	{
		// Mark EEPROM as faulty
		eeprom_update_byte ((uint8_t*) 0x00, 0xFF);

		//Set back lights to yellow, TC-channel to red
		PORTA = 0x77;	PORTB = 0x0C; PORTC = 0xB5;	PORTD = 0xD8;
		
		while(1);
	}
	sei();			
}

// Write mode to EEPROM
void SetMode(uint8_t mode)
{
	// Mode 0 = Normal Ambilight function, fade out when no input
	// Mode 1 = Store last input if no new data is introduced
	
	cli();
	eeprom_update_byte ((uint8_t*)	0x05, mode);	// Write Mode
	sei();
	
	// Set mode by changing the prefix in boblight settings, +1
	// 5A = normal, 5B = with color memory
}

// Read color-table from EEPROM
void ReadEE_Color(void)
{
	volatile uint8_t i;
	
	for(i = 1; i <= CHANNELS+1;i++)
	{
		Compare[i] = RXBuffer[i] = eeprom_read_byte((uint8_t)i+0xA0); 
	}
}

// Write current color-table to EEPROM
void WriteEE_Color(void)
{
	// Disable all interrupts
	cli();	
	
	uint8_t i;

	for(i = 1; i <= CHANNELS+1; i++)
	{
		// Run 4 PWM cycles to prevent blinking
		for(volatile int j = 0; j < 1024; j++) tickPWM();
		
		// Save one color component
		eeprom_update_byte ((uint8_t)i+0xA0, RXBuffer[i]);		
	}

	// Re-enable all interrupts
	sei();
}

// LiveLight Beach ball at first startup
void setWelcomeColor(void)
{
	RXBuffer[1]= 3;			RXBuffer[4] = 86;		RXBuffer[7] = 133;
	RXBuffer[2] = 247;		RXBuffer[5] = 254;		RXBuffer[8] = 254;
	RXBuffer[3] = 155;		RXBuffer[6] = 2;		RXBuffer[9] = 1;

	RXBuffer[10] = 254;		RXBuffer[13] = 254;		RXBuffer[16] = 254;
	RXBuffer[11] = 175;		RXBuffer[14] = 15;		RXBuffer[17] = 9;
	RXBuffer[12] = 1;		RXBuffer[15] = 21;		RXBuffer[18] = 70;
	
	RXBuffer[19] = 118;		RXBuffer[22] = 42;		RXBuffer[25] = 3;
	RXBuffer[20] = 1;		RXBuffer[23] = 28;		RXBuffer[26] = 221;
	RXBuffer[21] = 254;		RXBuffer[24] = 254;		RXBuffer[27] = 215;
}