/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
DSP effects for LiveLight V3 Firmware 						 		 ________   __
Written by Henri Rantanen											|  ____\ \ / /
																	| |__   \ V / 
Date		28.4.2014												|  __|   > <  
Version		3.0.1													| |     / . \ 
Target		PCB Version 5, Atmel ATMEGA32-16U						|_|    /_/ \_\

************************************************************************************/
#include "settings.h"

// Reset time to Off-line
void ResetShutdownTimer(void)					
{
	TCNT0 = 0;									// Reset Shut-Down timer
	TIMSK |= (1 << TOIE0);						// Re-enable shut-down timer after "Off-line"
}

// TIMER0 Overflow interrupt (Sleep counter)
ISR(TIMER0_OVF_vect)
{	
	// Off-line timer calculator
	static uint8_t NoInputCounter = 0;			
	
	// If counter has reached max. value
	if (NoInputCounter > EE_SLEEP_DELAY)		
	{
		// Disable TIMER0 interrupt
		TIMSK &= ~(1 << TOIE0);
// 		TIMSK |= (1 << TOIE2); // enable PWM	
// 		
// 		TCNT2 = 0;
// 		TCNT0 = 0;						
		
		// Reset shutdown counter
		NoInputCounter = 0;
		
		// Switch mode
		if (RXBuffer[0] == EE_PREFIX)
		{
			SetMode(0x00);		// Ambilight mode
			FadeOut();			// Fade lights OFF
			resetModule();
		}		
		if (RXBuffer[0] == EE_PREFIX + 1) 
		{
			SetMode(0x01);		// Static light mode
			WriteEE_Color();	// Save colors to EEPROM
			resetModule();
		}
	}
	
	else NoInputCounter++;						// Grow Off-line counter
}


// Set RGB color to registers
void Color(uint8_t Red, uint8_t Green, uint8_t Blue)
{
	// Browse through all RGB channels
	for(uint8_t i = 0; i < CHANNELS; i=i+3)		
	{
		RXBuffer[i+1] = Red;	// Set channels RED value
		RXBuffer[i+2] = Green;	// Set channels GREEN value
		RXBuffer[i+3] = Blue;	// Set channels BLUE value
	}
}


void FadeOut(void)
{
	// Set saturation to 0%
	/*for (i = 0; i < 9; i++)
	{		
		RXBuffer[i*3+1] = RXBuffer[i*3+2] = RXBuffer[i*3+3] = ((RXBuffer[i*3+1] + RXBuffer[i*3+2] + RXBuffer[i*3+3]) / 3);	
	}*/
	
	// Disable global PWM and other interrupts
	cli(); 
	
	// Dim all RGB channels from 255 to zero
	for (uint8_t i = 0; i < 255; i++)
	{
		// All channels brightness minus one
		for (uint8_t j = 0; j <= CHANNELS; j++)
		{
			if (RXBuffer[j] > 0) RXBuffer[j]--;
		}
		
		// Run 4 full PWM cycles
		for(uint16_t j = 0; j < 1024; j++) tickPWM();
	}
	
	// Set color to black, just in case...
	Color(0,0,0); 
	
	Flush_USART();
			
	// Re-enable interrupts
	sei();
}

// Do a hardware reset
void resetModule(void)
{
	// Disable all interrupts
	cli();
	
	// Set watchdoge timer to ~17 mS
	WDTCR = 0x08;
	
	// Run in a loop until watchdoge resets the device
	while(1);
}