/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
LiveLight USB Interface with ATMEGA32
Written by Henri Rantanen 

Date		28.4.2014
Version		3.0.1
Target		PCB Version 5, Atmel ATMEGA32-16U

************************************************************************************/
#include "settings.h"

int main(void)
{	 
	// Initialize the MCU
	Initialize();
	
	while(1)
    {
		// Main loop is empty
 		// Interrupts do all the work!
    }
}



void Initialize(void)
{
	// Set all output pins off
	DDRA = 0xFF;	PORTA = 0x00;					// PORT A 1111 1111
	DDRB = 0xFF;	PORTB = 0x00;					// PORT B 1111 1111
	DDRC = 0xFF;	PORTC = 0x00;					// PORT C 1111 1111
	DDRD = 0xFC;	PORTD = 0x00;					// PORT D 1111 1100
	// (For PCB V4) DDRD = 0xFE;					// PORT D 1111 1110

	// Run in first start after flashing the chip
	// Read first byte from EEPROM and check if memory is empty or damaged.
	if (EE_HEADER != HEADER)
	{
		// Write default settings to EEPROM	
		WriteEE_defaults();
		
		// Show the LiveLight rainbow
		setWelcomeColor();		
	}
	 
	// Read last used colors from EEPROM if in static color mode
	if (EE_MODE == 0x01) ReadEE_Color();
	
	
	// Initialize PWM Timer
	TCCR2 |= (0 << CS22)|(1 << CS21)|(0 << CS20);	// Timer pre-scaler
	TIMSK |= (1 << TOIE2);							// Enable Overflow Interrupt
	//TCNT2 = 0;									// Initialize Counter
	
	//Initialize USART
	UBRRH = 0;										// UART Speed HIGH
	UBRRL = ((F_CPU / 16) / (EE_BAUD * 100)) - 1; // UART Speed LOW
	UCSRB |= (1 << RXEN )|(1 << TXEN )|(1 << RXCIE);// Enable RX, TX and RX Interrupt
	UCSRC |= (1 << URSEL)|(1 << UCSZ1)|(1 << UCSZ0);// Character size 8 bits
		
	// Initialize sleep Timer
	TCCR0 |= (1 << CS02)|(0 << CS01)|(0 << CS00);	// Timer pre-scaler 1/1024
	//TIMSK |= (1 << TOIE0);							// Enable Overflow Interrupt
	//TCNT0 = 0;										// Initialize Counter	
	
	// Enable global interrupts and start operation
	sei();											
}
