/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
PWM Algorithm for LiveLight V3 Firmware					 _______          ____  __ 
Written by Henri Rantanen								|  __ \ \        / /  \/  |
														| |__) \ \  /\  / /| \  / |
Date		26.4.2014									|  ___/ \ \/  \/ / | |\/| |
Version		3.0											| |      \  /\  /  | |  | |
Target		PCB Version 5, Atmel ATMEGA32-16U			|_|       \/  \/   |_|  |_|

************************************************************************************/
#include "settings.h"

// TIMER2 Overflow interrupt
ISR(TIMER2_OVF_vect)
{
	tickPDM();		// Pulse Density Modulation
	TCNT2 = 200;	// Set the starting value of the PDM timer

	//tickPWM();		// Pulse Width Modulation
	//TCNT2 = 250;	// Set the starting value of the PWM timer
}

void tickPWM(void)
{
	static uint8_t PWM_Index = 0xFF;	// PWM Index counter
	static uint8_t PinMaskA = 0xFF;		// Port A Output Pattern
	static uint8_t PinMaskB = 0xFF;		// Port B Output Pattern
	static uint8_t PinMaskC = 0xFF;		// Port C Output Pattern
	static uint8_t PinMaskD = 0xFC;		// Port D Output Pattern
	
	// Update outputs
	PORTA = PinMaskA;					// Set A port to calculated pattern
	PORTB = PinMaskB;					// Set B port to calculated pattern
	PORTC = PinMaskC;					// Set C port to calculated pattern
	PORTD = PinMaskD;					// Set D port to calculated pattern
	
	if (++PWM_Index == 0)				// Update colors from RX buffer
	{
		PinMaskA = 0xFF;				// Set Port A ON
		PinMaskB = 0xFF;				// Set Port B ON
		PinMaskC = 0xFF;				// Set Port C ON
		PinMaskD = 0xFC;				// Set Port D ON except TX & RX pins
		
		Compare[1] = RXBuffer[1];		// Update channels from receiver buffer 
		Compare[2] = RXBuffer[2];
		Compare[3] = RXBuffer[3];
		Compare[4] = RXBuffer[4];
		Compare[5] = RXBuffer[5];
		Compare[6] = RXBuffer[6];
		Compare[7] = RXBuffer[7];
		Compare[8] = RXBuffer[8];		// No loop? This runs faster!
		Compare[9] = RXBuffer[9];
		Compare[10] = RXBuffer[10];
		Compare[11] = RXBuffer[11];
		Compare[12] = RXBuffer[12];
		Compare[13] = RXBuffer[13];
		Compare[14] = RXBuffer[14];
		Compare[15] = RXBuffer[15];
		Compare[16] = RXBuffer[16];
		Compare[17] = RXBuffer[17];
		Compare[18] = RXBuffer[18];
		Compare[19] = RXBuffer[19];
		Compare[20] = RXBuffer[20];
		Compare[21] = RXBuffer[21];
		Compare[22] = RXBuffer[22];
		Compare[23] = RXBuffer[23];
		Compare[24] = RXBuffer[24];
		Compare[25] = RXBuffer[25];
		Compare[26] = RXBuffer[26];
		Compare[27] = RXBuffer[27];
	}
		
	// Remove channels from the pattern used in next interrupt
	if(Compare[1] == PWM_Index)		PinMaskA &= ~(1 << PA2); // Red1
	if(Compare[2] == PWM_Index)		PinMaskA &= ~(1 << PA4); // Green1
	if(Compare[3] == PWM_Index)		PinMaskA &= ~(1 << PA3); // Blue1
	if(Compare[4] == PWM_Index)		PinMaskB &= ~(1 << PB2); // Red2
	if(Compare[5] == PWM_Index)		PinMaskA &= ~(1 << PA1); // Green2
	if(Compare[6] == PWM_Index)		PinMaskB &= ~(1 << PB1); // Blue2
	if(Compare[7] == PWM_Index)		PinMaskA &= ~(1 << PA0); // Red3
	if(Compare[8] == PWM_Index)		PinMaskB &= ~(1 << PB3); // Green3
	if(Compare[9] == PWM_Index)		PinMaskB &= ~(1 << PB0); // Blue3
	if(Compare[10] == PWM_Index)	PinMaskD &= ~(1 << PD3); // Red4
	if(Compare[11] == PWM_Index)	PinMaskB &= ~(1 << PB4); // Green4
	if(Compare[12] == PWM_Index)	PinMaskD &= ~(1 << PD2); // Blue4
	if(Compare[13] == PWM_Index)	PinMaskD &= ~(1 << PD4); // Red5
	if(Compare[14] == PWM_Index)	PinMaskD &= ~(1 << PD6); // Green5
	if(Compare[15] == PWM_Index)	PinMaskD &= ~(1 << PD5); // Blue5
	if(Compare[16] == PWM_Index)	PinMaskC &= ~(1 << PC0); // Red6
	if(Compare[17] == PWM_Index)	PinMaskD &= ~(1 << PD7); // Green6
	if(Compare[18] == PWM_Index)	PinMaskC &= ~(1 << PC1); // Blue6
	if(Compare[19] == PWM_Index)	PinMaskC &= ~(1 << PC4); // Red7
	if(Compare[20] == PWM_Index)	PinMaskC &= ~(1 << PC2); // Green7
	if(Compare[21] == PWM_Index)	PinMaskC &= ~(1 << PC3); // Blue7
	if(Compare[22] == PWM_Index)	PinMaskC &= ~(1 << PC5); // Red8
	if(Compare[23] == PWM_Index)	PinMaskC &= ~(1 << PC7); // Green8
	if(Compare[24] == PWM_Index)	PinMaskC &= ~(1 << PC6); // Blue8
	if(Compare[25] == PWM_Index)	PinMaskA &= ~(1 << PA5); // Red9
	if(Compare[26] == PWM_Index)	PinMaskA &= ~(1 << PA6); // Green9
	if(Compare[27] == PWM_Index)	PinMaskA &= ~(1 << PA7); // Blue9
}	

void tickPDM(void)
{
	// PDM variables
	static uint8_t step = 0;
	static int16_t error[27] = {128};
		
	if (step == 0)
	{
		// Load new color values from receive buffer and reset error counter
		for (uint8_t i = 0; i < 27; i++)
		{
			error[i] = 128;
			Compare[i+1] = RXBuffer[i+1];
		}
	}
	
	// Increment PDM step counter or reset it
	if (step != 255) step = step + 1;
	else step = 0;
	
	// Calculate the new error value
	for (uint8_t i = 0; i < 27; i++)
	{
		error[i] = error[i] - Compare[i+1];
	}


//****** Right Bottom channel ***************************************

	// Red 1
	if(error[0] < 0)
	{
		PORTA |= (1 << PA2);		// Turn channel ON
		error[0] = error[0] + 255;
	}
	else PORTA &= ~(1 << PA2);		// Turn channel OFF
	
	// Green 1
	if(error[1] < 0)
	{
		PORTA |= (1 << PA4);		
		error[1] = error[1] + 255;
	}
	else PORTA &= ~(1 << PA4);
	
	// Blue 1
	if(error[2] < 0)
	{
		PORTA |= (1 << PA3);
		error[2] = error[2] + 255;
	}
	else PORTA &= ~(1 << PA3);
	
//****** Right Top channel ***************************************

	// Red 2
	if(error[3] < 0)
	{
		PORTB |= (1 << PB2);
		error[3] = error[3] + 255;
	}
	else PORTB &= ~(1 << PB2);
	
	// Green 2
	if(error[4] < 0)
	{
		PORTA |= (1 << PA1);
		error[4] = error[4] + 255;
	}
	else PORTA &= ~(1 << PA1);
	
	// Blue 2
	if(error[5] < 0)
	{
		PORTB |= (1 << PB1);
		error[5] = error[5] + 255;
	}
	else PORTB &= ~(1 << PB1);
	
//****** Top Right channel ***************************************
	
	// Red 3
	if(error[6] < 0)
	{
		PORTA |= (1 << PA0);
		error[6] = error[6] + 255;
	}
	else PORTA &= ~(1 << PA0);
		
	// Green 3
	if(error[7] < 0)
	{
		PORTB |= (1 << PB3);
		error[7] = error[7] + 255;
	}
	else PORTB &= ~(1 << PB3);
		
	// Blue 3
	if(error[8] < 0)
	{
		PORTB |= (1 << PB0);
		error[8] = error[8] + 255;
	}
	else PORTB &= ~(1 << PB0);

//****** Top Center channel ***************************************	
	
	// Red 4
	if(error[9] < 0)
	{
		PORTD |= (1 << PD3);
		error[9] = error[9] + 255;
	}
	else PORTD &= ~(1 << PD3);
		
	// Green 4
	if(error[10] < 0)
	{
		PORTB |= (1 << PB4);
		error[10] = error[10] + 255;
	}
	else PORTB &= ~(1 << PB4);
		
	// Blue 4
	if(error[11] < 0)
	{
		PORTD |= (1 << PD2);
		error[11] = error[11] + 255;
	}
	else PORTD &= ~(1 << PD2);

//****** Top Left channel ***************************************

	// Red 5
	if(error[12] < 0)
	{
		PORTD |= (1 << PD4);
		error[12] = error[12] + 255;
	}
	else PORTD &= ~(1 << PD4);
	
	// Green 5
	if(error[13] < 0)
	{
		PORTD |= (1 << PD6);
		error[13] = error[13] + 255;
	}
	else PORTD &= ~(1 << PD6);
	
	// Blue 5
	if(error[14] < 0)
	{
		PORTD |= (1 << PD5);
		error[14] = error[14] + 255;
	}
	else PORTD &= ~(1 << PD5);
		
//****** Left Top channel ***************************************
		
	// Red 6
	if(error[15] < 0)
	{
		PORTC |= (1 << PC0);
		error[15] = error[15] + 255;
	}
	else PORTC &= ~(1 << PC0);
	
	// Green 6
	if(error[16] < 0)
	{
		PORTD |= (1 << PD7);
		error[16] = error[16] + 255;
	}
	else PORTD &= ~(1 << PD7);
	
	// Blue 6
	if(error[17] < 0)
	{
		PORTC |= (1 << PC1);
		error[17] = error[17] + 255;
	}
	else PORTC &= ~(1 << PC1);	
			
//****** Left Bottom channel ***************************************
			
	// Red 7
	if(error[18] < 0)
	{
		PORTC |= (1 << PC4);
		error[18] = error[18] + 255;
	}
	else PORTC &= ~(1 << PC4);
	
	// Green 7
	if(error[19] < 0)
	{
		PORTC |= (1 << PC2);
		error[19] = error[19] + 255;
	}
	else PORTC &= ~(1 << PC2);
	
	// Blue 7
	if(error[20] < 0)
	{
		PORTC |= (1 << PC3);
		error[20] = error[20] + 255;
	}
	else PORTC &= ~(1 << PC3);
		
//****** Bottom Left channel ***************************************
		
	// Red 8
	if(error[21] < 0)
	{
		PORTC |= (1 << PC5);
		error[21] = error[21] + 255;
	}
	else PORTC &= ~(1 << PC5);
			
	// Green 8
	if(error[22] < 0)
	{
		PORTC |= (1 << PC7);
		error[22] = error[22] + 255;
	}
	else PORTC &= ~(1 << PC7);
			
	// Blue 8
	if(error[23] < 0)
	{
		PORTC |= (1 << PC6);
		error[23] = error[23] + 255;
	}
	else PORTC &= ~(1 << PC6);
			
//****** Bottom Right channel ***************************************

	// Red 9
	if(error[24] < 0)
	{
		PORTA |= (1 << PA5);
		error[24] = error[24] + 255;
	}
	else PORTA &= ~(1 << PA5);
	
	// Green 9
	if(error[25] < 0)
	{
		PORTA |= (1 << PA6);
		error[25] = error[25] + 255;
	}
	else PORTA &= ~(1 << PA6);
	
	// Blue 9
	if(error[26] < 0)
	{
		PORTA |= (1 << PA7);
		error[26] = error[26] + 255;
	}
	else PORTA &= ~(1 << PA7);

}