/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
PWM Algorithm for LiveLight V2 Firmware					 _______          ____  __ 
Written by Henri Rantanen								|  __ \ \        / /  \/  |
														| |__) \ \  /\  / /| \  / |
DDate		6.5.2012									|  ___/ \ \/  \/ / | |\/| |
Version		2.5											| |      \  /\  /  | |  | |
Target		PCB Version 5, Atmel ATMEGA32-16U			|_|       \/  \/   |_|  |_|

************************************************************************************/
#include "settings.h"

// TIMER2 Overflow interrupt
ISR(TIMER2_OVF_vect)
{
	switch_routine();
	
	TCNT2 = 200;	// Set the starting value of the PWM timer	
}

void switch_routine(void)
{
	static uint8_t PWM = 0;
	static uint8_t flip = 0;
	
	if (flip == 0)
	{
		PWM++;
	} 
	else
	{
		PWM--;
	}
	
	flip = ~(flip);
	PWM = ~(PWM);
	
	
// Right Bottom RGB RXBuffer
		if(RXBuffer[1] > PWM)  		// 1. RXBuffer switch
		{
			PORTA |= 0x04;
		}
		else
		{
			PORTA = PORTA & ~0x04;
		}

		if(RXBuffer[2] > PWM) 		// 2. RXBuffer switch
		{
			PORTA |= 0x10;
		}
		else
		{
			PORTA = PORTA & ~0x10;
		}

		if(RXBuffer[3] > PWM) 		// 3. RXBuffer switch
		{
			PORTA |= 0x8;
		}
		else
		{
			PORTA = PORTA & ~0x8;
		}

// Right Top RGB RXBuffer
		if(RXBuffer[4] > PWM) 		// 4. RXBuffer switch
		{
			PORTB |= 0x04;
		}
		else
		{
			PORTB = PORTB & ~0x04;
		}
		if(RXBuffer[5] > PWM) 		// 5. RXBuffer switch
		{
			PORTA |= 0x02; 
		}
		else
		{
			PORTA = PORTA & ~0x02;
		}
		
		if(RXBuffer[6] > PWM) 		// 6. RXBuffer switch
		{
			PORTB |= 0x02; 
		}
		else
		{
			PORTB = PORTB & ~0x02;
		}
	
// Top Right RGB RXBuffer
		if(RXBuffer[7] > PWM) 		// 7. RXBuffer switch
		{
			PORTA |= 0x01;
		}
		else
		{
			PORTA = PORTA & ~0x01;
		}
		
		if(RXBuffer[8] > PWM) 		// 8. RXBuffer switch
		{
			PORTB |= 0x08;
		}
		else
		{
			PORTB = PORTB & ~0x08;
		}
		
		if(RXBuffer[9] > PWM) 		// 9. RXBuffer switch
		{
			PORTB |= 0x01;
		}		
		else
		{
			PORTB = PORTB & ~0x01;
		}


// Top Center RGB RXBuffer
		if(RXBuffer[10] > PWM) 		// 10. RXBuffer switch
		{
			PORTD |= 0x08;
		}
		else
		{
			PORTD = PORTD & ~0x08;
		}
		
		if(RXBuffer[11] > PWM) 		// 11. RXBuffer switch
		{
			PORTD |= 0x02;
		}
		else
		{
			PORTD = PORTD & ~0x02;
		}
		
		if(RXBuffer[12] > PWM)		// 12. RXBuffer switch
		{
			PORTD |= 0x04;
		}
		else
		{
			PORTD = PORTD & ~0x04;
		}

// Top Left RGB RXBuffer
		if(RXBuffer[13] > PWM) 		// 13. RXBuffer switch
		{
			PORTD |= 0x10;
		}
		else
		{
			PORTD = PORTD & ~0x10;
		}
		
		if(RXBuffer[14] > PWM) 		// 14. RXBuffer switch
		{
			PORTD |= 0x40;
		}
		else
		{
			PORTD = PORTD & ~0x40;
		}

		if(RXBuffer[15] > PWM)		// 15. RXBuffer switch
		{
			PORTD |= 0x20;
		}
		else
		{
			PORTD = PORTD & ~0x20;
		}

// Left Top RGB RXBuffer
		if(RXBuffer[16] > PWM) 		// 16. RXBuffer switch
		{
			PORTC |= 0x01;
		}
		else
		{
			PORTC = PORTC & ~0x01;
		}
		
		if(RXBuffer[17] > PWM) 		// 17. RXBuffer switch
		{
			PORTD |= 0x80;
		}
		else
		{
			PORTD = PORTD & ~0x80;
		}

		if(RXBuffer[18] > PWM)		// 18. RXBuffer switch
		{
			PORTC |= 0x02;
		}
		else
		{
			PORTC = PORTC & ~0x02;
		}
// Left Bottom RGB RXBuffer

		if(RXBuffer[19] > PWM) 		// 19. RXBuffer switch
		{
			PORTC |= 0x10;
		}
		else
		{
			PORTC = PORTC & ~0x10;
		}
		
		if(RXBuffer[20] > PWM) 		// 20. RXBuffer switch
		{
			PORTC |= 0x04;
		}
		else
		{
			PORTC = PORTC & ~0x04;
		}
		
		if(RXBuffer[21] > PWM)		// 21. RXBuffer switch
		{
			PORTC |= 0x08;
		}
		else
		{
			PORTC = PORTC & ~0x08;
		}
		
// Bottom Left RGB RXBuffer
		if(RXBuffer[22] > PWM) 		// 22. RXBuffer switch
		{
			PORTC |= 0x20;
		}
		else
		{
			PORTC = PORTC & ~0x20;
		}
		
		if(RXBuffer[23] > PWM) 		// 23. RXBuffer switch
		{
			PORTC |= 0x80;
		}
		else
		{
			PORTC = PORTC & ~0x80;
		}
		
		if(RXBuffer[24] > PWM)		// 24. RXBuffer switch
		{
			PORTC |= 0x40;
		}
		else
		{
			PORTC = PORTC & ~0x40;
		}	
		
//Bottom Right RGB RXBuffer
	

		if(RXBuffer[25] > PWM)		// 25. RXBuffer switch
		{
			PORTA |= 0x20;
		}
		else
		{
			PORTA = PORTA & ~0x20;
		}

		if(RXBuffer[26] > PWM)		// 26. RXBuffer switch
		{
			PORTA |= 0x40;
		}
		else
		{
			PORTA = PORTA & ~0x40;
		}

		if(RXBuffer[27] > PWM)		// 27. RXBuffer switch
		{
			PORTA |= 0x80;
		}
		else
		{
			PORTA = PORTA & ~0x80;
		}
}
