/************************************************************************************
					 _     _           _     _       _     _   
					| |   (_)         | |   (_)     | |   | |  
					| |    ___   _____| |    _  __ _| |__ | |_ 
					| |   | \ \ / / _ \ |   | |/ _` | '_ \| __|
					| |___| |\ V /  __/ |___| | (_| | | | | |_ 
					\_____/_| \_/ \___\_____/_|\__, |_| |_|\__|
												__/ |          
BETA Firmware 								   |___/
							
Settings file for LiveLight V3 Firmware
Written by Henri Rantanen 

Date		10.5.2014
Version		3.0.2
Target		PCB Version 5, Atmel ATMEGA32-16U

************************************************************************************/

#ifndef SETTINGS_H_
#define SETTINGS_H_

// Module parameters
#define F_CPU 			14745600	// Clock speed in Hz
#define CHANNELS		27			// Amount of channels
#define HEADER			0xDB		// EEPROM Check byte
#define BAUD			9600		// Set BAUD-rate
#define PREFIX			0x5A		// Header byte for data transfer
#define SLEEP_DELAY		40			// How long to wait for data until going to sleep
#define BLACKLEVEL		1			// Minimum brightness 0-255
#define MODE			0			// 0 = Ambilight, 1 = Static

// Define shortcuts for reading EEPROM
#define EE_HEADER		eeprom_read_byte(0x00)	// Read EEPROM Header byte
#define	EE_BAUD			eeprom_read_byte(0x01)	// Read BAUD-rate
#define EE_PREFIX		eeprom_read_byte(0x02)	// Read Header byte
#define EE_SLEEP_DELAY	eeprom_read_byte(0x03)	// Read SLEEP_DELAY
#define EE_BLACK_LEVEL	eeprom_read_byte(0x04)	// Read Minimum brightness
#define EE_MODE			eeprom_read_byte(0x05)	// Read current Mode
										
// Header files
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <stdlib.h>

// Global variables
volatile uint8_t Compare  [1 + CHANNELS];		// Stores all the brightness settings in use
volatile uint8_t RXBuffer [1 + CHANNELS];		// USART Receiver buffer

// Function prototypes
void ResetShutdownTimer(void);							// Resets countdown to off-line mode
void TX_Char(volatile uint8_t data);					// Transmits character to PC
void Initialize(void);									// Initializes the C
void Color(uint8_t Red, uint8_t Green, uint8_t Blue);	// Status Output (RGB)
void FadeOut(void);										// Fade-out all channels
void tickPWM(void);										// Pulse Width Modulation switching cycle
void tickPDM(void);										// Pulse Density Modulation switching cycle
void WriteEE_defaults(void);							// Write default settings to EEPROM
uint8_t Average(uint8_t old, uint8_t new);				// Calculate moving average of the input
void SetMode(uint8_t i);								// Save mode
void ReadEE_Color(void);								// Read saved colors from EEPROM
void WriteEE_Color(void);								// Save current colors to EEPROM
void Flush_USART(void);
void setWelcomeColor(void);
void resetModule(void);

#endif /* SETTINGS_H_ */

/********* EEPROM ***********
|  Slot	|	Setting			|
|-------|-------------------|
|	0	|	Check byte		|
|	1	|	BAUD_RATE		|
|	2	|	HEADER_BYTE		|
|	3	|	SLEEP_DELAY		|
|	4	|	BLACK_LEVEL		|
|	5	|	MODE			|
|	6	|	CH1 color		|
|	7	|	CH2 color		|
|   --  |   --				|
|	33	|	CH27 color		|
****************************/